# -*- coding: utf8 -*-
from __future__ import unicode_literals

import ftplib
import io
import os
import time

from driverng.transport.base import (
    BaseFileBackend,
    BaseFileTransport,
    TransportException,
)


def catch_ftp_exception(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except ftplib.all_errors as e:
            raise TransportException(e)

    return wrapper


class FTP(BaseFileBackend):
    def __init__(
        self,
        hostname=None,
        port=None,
        username=None,
        password=None,
        mode="active",
        transfert_mode="binary",
        logger=None,
        del_command_disable=False,
        force_close=False,
    ):
        self.hostname = hostname
        self.username = username
        self.password = password
        self.port = port
        self.mode = mode
        self.transfert_mode = transfert_mode
        self._ttl = time.time()
        self._ftp = None
        self.logger = logger
        self.del_command_disable = del_command_disable
        self.force_close = force_close

    @property
    def _expired(self):
        return (time.time() - self._ttl) > 120

    @catch_ftp_exception
    def _connection(self):
        if self._ftp is None:
            self._ftp = ftplib.FTP()
            self._ftp.connect(self.hostname, self.port, 30)
            self._ftp.login(self.username, self.password)
            self._ftp.set_pasv(self.mode == "passive")
            self.logger.info(
                "Connected to ftp://{}@{}:{}".format(
                    self.username, self.hostname, self.port
                )
            )
            self._ttl = time.time()

    @catch_ftp_exception
    def _deconnection(self, force_close):
        if force_close and self._ftp is not None:
            self._ftp.close()
            self.logger.info(
                "Closed ftp://{}:{}".format(self._ftp.host, self._ftp.port)
            )
            self._ftp = None
            self._ttl = 0

    @catch_ftp_exception
    def _reinit_connection(self):
        if self._ftp is not None:
            self._deconnection(True)

        self._connection()

    @property
    def ftp(self):
        if self._ftp is None or self._expired:
            self._reinit_connection()
        return self._ftp

    @catch_ftp_exception
    def cwd(self, path):
        self._connection()
        self.ftp.cwd(path)
        self.logger.info("Directory changed to {}".format(path))
        self._deconnection(self.force_close)

    @catch_ftp_exception
    def list(self, path=None):
        self._connection()
        files = []
        self.logger.info("List files on path '{}'".format(path))
        for filepath in self.ftp.nlst(path):
            files.append(os.path.basename(filepath))
        self._deconnection(self.force_close)
        return files

    @catch_ftp_exception
    def put(self, data, filename):
        self._connection()
        if self.transfert_mode == "ascii":
            self.logger.info("Put file '{}' Mode 'ascii'".format(filename))
            func = self.ftp.storlines
        else:
            self.logger.info("Put file '{}' Mode 'binary'".format(filename))
            func = self.ftp.storbinary
        result = func("STOR " + filename, io.BytesIO(data))
        self._deconnection(self.force_close)
        return result

    @catch_ftp_exception
    def get(self, filename):
        self._connection()
        if self.transfert_mode == "ascii":
            self.logger.info("Get file '{}' Mode 'ascii'".format(filename))
            func = self.ftp.retrlines
        else:
            self.logger.info("Get file '{}' Mode 'binary'".format(filename))
            func = self.ftp.retrbinary
        buf = io.BytesIO()
        func("RETR " + filename, buf.write)
        result = buf.getvalue()
        self._deconnection(self.force_close)
        return result

    @catch_ftp_exception
    def delete(self, filename):
        self._connection()
        if not self.del_command_disable:
            result = self.ftp.delete(filename)
            self.logger.info("Deleted file '{}'".format(filename))
            return result
        else:
            self.logger.info(
                "Not deleted file '{}' (del_command_disable=True)".format(filename)
            )
        self._deconnection(self.force_close)


class FTPTransport(BaseFileTransport):
    def __init__(
        self,
        hostname=None,
        username=None,
        port=None,
        password=None,
        mode="active",
        transfert_mode="binary",
        host=None,
        **kwargs
    ):
        del_command_disable = kwargs.pop("del_command_disable", False)
        force_close = kwargs.pop("force_close", False)
        if host is not None and hostname is None and port is None:
            if ":" in host:
                hostname, port = host.rsplit(":", 1)
                port = int(port)
            else:
                hostname = host
        backend = FTP(
            hostname or "localhost",
            port or 21,
            username or "anonymous",
            password,
            mode,
            transfert_mode,
            kwargs.get("logger"),
            del_command_disable,
            force_close,
        )
        super(FTPTransport, self).__init__(backend, **kwargs)
